@@ -1,7 +1,7 @@
 package WWW::PAUSE::Simple;
 
-our $DATE = '2015-06-14'; # DATE
-our $VERSION = '0.22'; # VERSION
+our $DATE = '2015-07-07'; # DATE
+our $VERSION = '0.23'; # VERSION
 
 use 5.010001;
 use strict;
@@ -85,6 +85,17 @@ $SPEC{':package'} = {
     summary => 'An API for PAUSE',
 };
 
+sub _parse_release_filename {
+    my $filename = shift;
+    return undef unless
+        $filename =~ /\A
+                      (\w+(?:-\w+)*)
+                      -v?(\d+(?:\.\d+){0,2}(_\d+|-TRIAL)?)
+                      \.$re_archive_ext
+                      \z/ix;
+    return ($1, $2, $3); # (dist, version, is_dev)
+}
+
 sub _common_args {
     my $args = shift;
     (username=>$args->{username}, password=>$args->{password});
@@ -112,9 +123,12 @@ sub _htres2envres {
     [$res->code, $res->message, $res->content];
 }
 
-$SPEC{upload_file} = {
+$SPEC{upload_files} = {
     v => 1.1,
     summary => 'Upload file(s) to your PAUSE account',
+    args_rels => {
+        choose_one => [qw/delay group_delay/],
+    },
     args => {
         %common_args,
         %files_arg,
@@ -133,22 +147,76 @@ currently might choke with SQLite database locking problem and thus fail to
 index your releases. Giving a delay of say 2-3 minutes (120-180 seconds) between
 files will alleviate this problem.
 
+_
+        },
+        group_delay => {
+            summary => 'Pause a number of seconds between groups of files',
+            schema => ['int*', min=>0],
+            description => <<'_',
+
+As an alternative to the `delay` option, you can also use this option. This will
+group the files to be uploaded by versions and prevent files of the same dist
+and different versions to be uploaded too close to one another, as this might
+cause indexing problem too. For example, suppose you're uploading Foo-1.zip
+Foo-2.zip Foo-3.zip Bar-1.zip Baz-1.zip Baz-2.zip. The files will be uploaded in
+this order:
+
+    upload: Foo-1.zip Bar-1.zip Baz-1.zip
+    group delay
+    upload: Foo-2.zip Baz-2.zip
+    group delay
+    upload: Foo-3.zip
+
+As with the `delay` option, it is recommended that if you upload several files
+at once, you set this option to 2-3 minutes (120-180 seconds).
+
 _
         },
     },
+    features => {dry_run=>1},
 };
-sub upload_file {
+sub upload_files {
     require File::Basename;
 
     my %args = @_;
-    my $files  = $args{files}
+    my $files0 = $args{files}
         or return [400, "Please specify at least one file"];
     my $subdir = $args{subdir} // '';
 
+    # probably group it into versions
+    my @files;
+    if ($args{group_delay}) {
+        my %files_by_dist; # key=distname, val=[file, ...]
+        my $max_num_vers = 0;
+        for my $file (sort @$files0) {
+            my ($dist, $ver, $is_dev) = _parse_release_filename($file);
+            if (!$dist) {
+                ($dist, $ver) = ("", 0);
+            }
+            $files_by_dist{$dist} //= [];
+            push @{ $files_by_dist{$dist} }, $file;
+            $max_num_vers = @{ $files_by_dist{$dist} }
+                if $max_num_vers < @{ $files_by_dist{$dist} };
+        }
+        for my $i (1..$max_num_vers) {
+            for my $dist (keys %files_by_dist) {
+                next unless @{ $files_by_dist{$dist} } >= $i;
+                push @files, {
+                    group => $i,
+                    file => $files_by_dist{$dist}[$i-1],
+                };
+            }
+        }
+    } else {
+        @files = map { +{group=>1, file=>$_} } @$files0;
+    }
+
     my $envres = envresmulti();
 
     my $i = 0;
-    for my $file (@$files) {
+    my $prev_group = 0;
+    for my $ent (@files) {
+        my $file = $ent->{file};
         my $res;
         {
             unless (-f $file) {
@@ -156,6 +224,11 @@ sub upload_file {
                 last;
             }
 
+            if ($args{-dry_run}) {
+                $log->tracef("[dry-run] Uploading %s ...", $file);
+                goto DELAY;
+            }
+
             $log->tracef("Uploading %s ...", $file);
             my $httpres = _request(
                 %args,
@@ -192,14 +265,34 @@ sub upload_file {
         $log->tracef("Result of upload: %s", $res);
         $envres->add_result($res->[0], $res->[1], $res->[3]);
 
-        if ($args{delay} && ++$i < @$files) {
-            $log->tracef("Sleeping between flies for %d second(s) ...", $args{delay});
-            sleep $args{delay};
+      DELAY:
+        {
+            # it's the last file, no point in delaying, just exit
+            last if ++$i >= @files;
+            if ($args{delay}) {
+                $log->tracef("Sleeping between files for %d second(s) ...", $args{delay});
+                sleep $args{delay};
+                last;
+            }
+            if ($args{group_delay} &&
+                    $ent->{group} > 1 && $prev_group != $ent->{group}) {
+                $log->tracef("Sleeping between group for %d second(s) ...", $args{group_delay});
+                sleep $args{group_delay};
+                $prev_group = $ent->{group};
+            }
         }
     }
     $envres->as_struct;
 }
 
+# old name, deprecated
+{
+    no strict 'refs';
+    *upload_file = \&upload_files;
+    $SPEC{upload_file} = { %{ $SPEC{upload_files} } }; # shallow clone
+    $SPEC{upload_file}{'x.no_index'} = 1;
+}
+
 $SPEC{list_files} = {
     v => 1.1,
     summary => 'List files on your PAUSE account',
@@ -348,15 +441,10 @@ sub list_dists {
             $log->debugf("Skipping %s: under a subdirectory", $file);
             next;
         }
-        unless ($file =~ /\A
-                          (\w+(?:-\w+)*)
-                          -v?(\d+(?:\.\d+){0,2}(_\d+|-TRIAL)?)
-                          \.$re_archive_ext
-                          \z/ix) {
+        my ($dist, $version, $is_dev) = _parse_release_filename($file) or do {
             $log->debugf("Skipping %s: doesn't match release regex", $file);
             next;
-        }
-        my ($dist, $version, $is_dev) = ($1, $2, $3);
+        };
         next if $is_dev && $newest_n;
         push @dists, {
             name => $dist,
@@ -625,7 +713,7 @@ WWW::PAUSE::Simple - An API for PAUSE
 
 =head1 VERSION
 
-This document describes version 0.22 of WWW::PAUSE::Simple (from Perl distribution WWW-PAUSE-Simple), released on 2015-06-14.
+This document describes version 0.23 of WWW::PAUSE::Simple (from Perl distribution WWW-PAUSE-Simple), released on 2015-07-07.
 
 =head1 SYNOPSIS
 
@@ -996,10 +1084,13 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 upload_file(%args) -> [status, msg, result, meta]
+=head2 upload_files(%args) -> [status, msg, result, meta]
 
 Upload file(s) to your PAUSE account.
 
+This function supports dry-run operation.
+
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -1017,6 +1108,26 @@ files will alleviate this problem.
 
 File names/wildcard patterns.
 
+=item * B<group_delay> => I<int>
+
+Pause a number of seconds between groups of files.
+
+As an alternative to the C<delay> option, you can also use this option. This will
+group the files to be uploaded by versions and prevent files of the same dist
+and different versions to be uploaded too close to one another, as this might
+cause indexing problem too. For example, suppose you're uploading Foo-1.zip
+Foo-2.zip Foo-3.zip Bar-1.zip Baz-1.zip Baz-2.zip. The files will be uploaded in
+this order:
+
+ upload: Foo-1.zip Bar-1.zip Baz-1.zip
+ group delay
+ upload: Foo-2.zip Baz-2.zip
+ group delay
+ upload: Foo-3.zip
+
+As with the C<delay> option, it is recommended that if you upload several files
+at once, you set this option to 2-3 minutes (120-180 seconds).
+
 =item * B<password>* => I<str>
 
 PAUSE password.
@@ -1031,6 +1142,16 @@ PAUSE ID.
 
 =back
 
+Special arguments:
+
+=over 4
+
+=item * B<-dry_run> => I<bool>
+
+Pass -dry_run=>1 to enable simulation mode.
+
+=back
+
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -1042,6 +1163,8 @@ that contains extra information.
 
 Return value:  (any)
 
+=for Pod::Coverage ^(upload_file)$
+
 =head1 SEE ALSO
 
 L<CPAN::Uploader> which also does uploading from CLI.
